/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util.swing.beanbinding;

import java.util.Date;
import org.jdesktop.beansbinding.Binding;
import org.jdesktop.beansbinding.Validator;
import org.openide.util.NbBundle;

/**
 * Validator which tests if the date of the first supplied date picker is before
 * the last supplied. Or if the end date optional parameter is set to true will
 * test if the startDate is set and the end date is not. If both are set will test as described above.
 *
 *
 * Will set the {@link Binding#targetNullValue} to to the last known value to
 * ensure that the source value is not updated when value is null. This is done
 * because the binding framework does not validate null. Will set the dates
 * again on the date pickers if they are correct the following case describes
 * the reason
 *
 * Both date pickers are filled and the user changes the start date to a date
 * after the end date. This is not valid and the changes will not be forwarded
 * to the bean. When the user then changes the end date to a valid date the
 * start date is still not forwarded but is valid
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages("ValidityDateValidator error message=Validity start should be before validity end")
public class ValidityDateValidator extends Validator<Date> {

    private Binding startDateBinding;
    private Binding endDateBinding;
    private boolean internalSave = false;
    private Date startDateNullValue;
    private Date endDateNullValue;
    private boolean endDateOptional;

    public ValidityDateValidator(Binding startDateBinding, Binding endDateBinding, boolean endDateOptional) {
        this.startDateBinding = startDateBinding;
        this.endDateBinding = endDateBinding;
        startDateNullValue = (Date) startDateBinding.getSourceProperty().getValue(startDateBinding.getSourceObject());
        endDateNullValue = (Date) endDateBinding.getSourceProperty().getValue(endDateBinding.getSourceObject());
        startDateBinding.setTargetNullValue(startDateNullValue);
        endDateBinding.setTargetNullValue(endDateNullValue);
        this.endDateOptional = endDateOptional;
    }

    @Override
    public Validator.Result validate(Date value) {
        if (!internalSave) {
            Date startDate = (Date) startDateBinding.getTargetProperty().getValue(startDateBinding.getTargetObject());
            Date endDate = (Date) endDateBinding.getTargetProperty().getValue(endDateBinding.getTargetObject());
            if (endDateOptional && startDate != null && endDate == null) {
                return null;
            } else if (startDate != null && endDate != null) {
                if (startDate.before(endDate)) {
                    //Valid so persist the date that is not validated
                    //There is a value set and the null value must be updated
                    //This ensures that the last known value is set when null is entered
                    if (value.equals(startDate)) {
                        if (startDateNullValue == null) {
                            startDateNullValue = new Date();
                        }
                        startDateNullValue.setTime(startDate.getTime());
                        //End date needs persisting
                        internalSave = true;
                        endDateBinding.save();
                    } else if (value.equals(endDate)) {
                        if (endDateNullValue == null) {
                            endDateNullValue = new Date();
                        }
                        endDateNullValue.setTime(endDate.getTime());
                        internalSave = true;
                        startDateBinding.save();
                    }
                    return null;
                }
            }
        } else {
            internalSave = false;
            return null;
        }
        return new Validator.Result(null, Bundle.ValidityDateValidator_error_message());
    }
}